// (C) 2019-2025 GoodData Corporation
import isEmpty from "lodash/isEmpty.js";
import { AnalyticalWorkspaceDecorator } from "./analyticalWorkspace.js";
import { OrganizationDecorator } from "./organization.js";
import { OrganizationsDecorator } from "./organizations.js";
class BackendWithDecoratedServices {
    capabilities;
    config;
    decorated;
    factories;
    constructor(backend, factories = {}) {
        this.decorated = backend;
        this.factories = factories;
        this.capabilities = backend.capabilities;
        this.config = backend.config;
    }
    authenticate(force) {
        return this.decorated.authenticate(force);
    }
    deauthenticate(returnTo) {
        return this.decorated.deauthenticate(returnTo);
    }
    isAuthenticated() {
        return this.decorated.isAuthenticated();
    }
    onHostname(hostname) {
        return new BackendWithDecoratedServices(this.decorated.onHostname(hostname), this.factories);
    }
    withAuthentication(provider) {
        return new BackendWithDecoratedServices(this.decorated.withAuthentication(provider), this.factories);
    }
    withTelemetry(componentName, props) {
        return new BackendWithDecoratedServices(this.decorated.withTelemetry(componentName, props), this.factories);
    }
    withCorrelation(correlationMetadata) {
        return new BackendWithDecoratedServices(this.decorated.withCorrelation(correlationMetadata), this.factories);
    }
    organization(organizationId) {
        return new OrganizationDecorator(this.decorated.organization(organizationId), this.factories);
    }
    organizations() {
        return new OrganizationsDecorator(this.decorated.organizations(), this.factories);
    }
    currentUser() {
        return this.decorated.currentUser();
    }
    workspace(id) {
        return new AnalyticalWorkspaceDecorator(this.decorated.workspace(id), this.factories);
    }
    entitlements() {
        return this.decorated.entitlements();
    }
    dataSources() {
        return this.decorated.dataSources();
    }
    workspaces() {
        return this.decorated.workspaces();
    }
}
/**
 * Decorated backend is a wrapper of any other backend implementations that can be used to enrich
 * functionality of the services that the wrapped backend normally provides.
 *
 * It can be for instance used to decorate execution factories and in conjunction with {@link DecoratedPreparedExecution}
 * also create decorated prepared executions.
 *
 * @param backend - instance of backend to decorate
 * @param decorators - configuration for the decorations
 * @returns new decorated backend
 * @alpha
 */
export function decoratedBackend(backend, decorators) {
    if (isEmpty(decorators)) {
        return backend;
    }
    return new BackendWithDecoratedServices(backend, decorators);
}
//# sourceMappingURL=index.js.map